/*
    FEARLUS model0-6-8+ELMM0-2: LandAllocator.m
    Copyright (C) 1999-2002  Macaulay Institute

    This file is part of FEARLUS model0-6-8+ELMM0-2, an agent-based model of land use
    change.

    FEARLUS model0-6-8+ELMM0-2 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FEARLUS model0-6-8+ELMM0-2 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
/*

    FEARLUS model0-6-8+ELMM0-2 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FEARLUS model0-6-8+ELMM0-2 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
/*


Implementation of the LandAllocator class.

*/

#import "LandAllocator.h"
#import "LandManager.h"
#import "Environment.h"
#import "LandParcel.h"
#import "FearlusArguments.h"
#import "Parameter.h"
#import "SubPopulation.h"
#import "AssocArray.h"
#import "Tuple.h"
#import "MiscFunc.h"		// shuffleList:

@implementation LandAllocator


/*

setMinColour:Max:

Set the maximum and minimum colours that can be allocated to land managers.
This is an observer thing, I know, and hence bad practice, but we'll have to
live with it, because I can't think of a better way to do it.

*/

-(void)setMinColour: (int)min max: (int)max {
  minColour = min;
  maxColour = max;
}

/*

setParametersInitStrategyMax:initStrategyMin:strategyUnit:
landUseChangeYieldThreshold:transferLandUseYieldThreshold:neighbourMutateMax:
neighbourMutateMin:

Set the model parameters appropriate to the land allocator.

*/

-(void)setParameters: (Parameter *)p {
  parameter = p;
  landManagers = [List create: [self getZone]];
  deadLandManagers = [List create: [self getZone]];
  justDeadLandManagers = [List create: [self getZone]];
  deadDeadLandManagers = [List create: [self getZone]];
  parcelsForSale = [List create: [self getZone]];
  bids = [[[AssocArray createBegin: [self getZone]]
	    setSize: (([p environmentXSize] * [p environmentYSize]) / 4) + 1]
	   createEnd];
}

/*

initialiseWithEnvironment

Method to initialise the land allocator object. Passes in the
environment. Sets up the land manager lists, creates the land
managers.

*/

-(void)initialiseWithEnvironment: (Environment *)e {
  int maxx, maxy, x, y;
  unsigned nlpx, nlpy;
  LandParcel *lp;
  LandManager *lm;

  // Set all the parameters for the land allocator
  nextColour = minColour;
  environment = e;
  // For each cell in the environment, create a land manager to manage it
  maxx = [environment getSizeX];
  maxy = [environment getSizeY];
  nlpx = [parameter nInitXParcels];
  nlpy = [parameter nInitYParcels];
  if(maxx % nlpx != 0) {
    fprintf(stderr, "Environment X dimension %d not divisible by the "
	    "number of initial land parcels on the X axis to allocate to land "
	    "managers %u\n", maxx, nlpx);
    exit(1);
  }
  if(maxy % nlpy != 0) {
    fprintf(stderr, "Environment Y dimension %d not divisible by the "
	    "number of initial land parcels on the Y axis to allocate to land "
	    "managers %u\n", maxy, nlpy);
    exit(1);
  }
  for(x = 0; x < maxx / nlpx; x++) {
    for(y = 0; y < maxy / nlpy; y++) {
      int x1, y1;

      lm = [self createNewLandManager];
      if([FearlusArguments debugAllocator]) {
	printf("A: Created land manager ID %u\n", [lm getPIN]);
      }
      for(x1 = 0; x1 < nlpx; x1++) {
	for(y1 = 0; y1 < nlpy; y1++) {
	  lp = [environment getObjectAtX: ((x * nlpx) + x1)
			    Y: ((y * nlpy) + y1)];
	  [lm addLandParcel: lp price: 0.0];
				// Allocate the parcel to the manager -- the
				// initial parcels are free
	  [lp setLandManager: lm];
				// Tell the parcel who its manager is
	  if([FearlusArguments debugAllocator]) {
	    printf("A: Assigned land parcel %u at (%d, %d) to land manager"
		   "ID %u\n", [lp getPIN], ((x * nlpx) + x1),
		   ((y * nlpy) + y1), [lm getPIN]);
	  }
	}
      }
    }
  }
}

/*

addLandParcelForSale:

Add a land parcel to the list of land parcels for sale. This method is called
from land managers when they are selling off land parcels in order to get out
of the red.

*/

-(void)addLandParcelForSale: (LandParcel *)lp {
  [parcelsForSale addLast: lp];
}

/*

notifyParcelSales

Notify neighbouring land managers of land parcels for sale

*/

-(void)notifyParcelSales {
  id <Index> ix;
  LandParcel *lp;

  for(ix = [parcelsForSale begin: scratchZone], lp = [ix next];
      [ix getLoc] == Member;
      lp = [ix next]) {
    id <Index> nix;
    LandParcel *nlp;

    for(nix = [lp nbrBegin: scratchZone], nlp = [nix next];
	[nix getLoc] == Member;
	nlp = [nix next]) {
      LandManager *lm;

      lm = [nlp getLandManager];
      if([lm eligibleForNewLandParcels]) {
	[lm parcelAvailable: lp];
      }
    }
    [nix drop];
  }
  [ix drop];
}

/*

manager:bids:forParcel:

ELMM 0-1: Record a bid for a land parcel. Ignore a bid with non-positive price

*/

-(void)manager: (LandManager *)lm 
	  bids: (double)price
     forParcel: (LandParcel *)lp {
  Tuple *bid;
  id <List> lp_bids;

  if(price <= 0.0) return;
  bid = [[[Tuple create: scratchZone] setObj: lm] setDouble: price];
  lp_bids = [bids getObjectWithKey: lp];
  if(lp_bids == nil) {
    lp_bids = [List create: scratchZone];
    [bids addObject: lp_bids withKey: lp];
  }
  [lp_bids addLast: bid];
  [[lm getSubPopulation] addBid: price];
}

/*

transferLandParcels

First shuffle the list of land parcels for sale using a list-shuffler
object.  Then find land managers to buy each of the land parcels. A
Land Manager may buy a land parcel if they have enough wealth to cover
the landParcelPrice without going into the red. There is always a
chance of creating a new land manager to own the parcel.

ELMM 0-1, use a Vickrey auction, in which the highest bidder wins, but
pays the second-highest bid as price.

*/

-(void)transferLandParcels {
  while([parcelsForSale getCount] > 0) {
    LandParcel *lp;
    id <List> lp_bids;
    SubPopulation *new_sp = [self chooseSubPopulation];
    double new_bid;
    Tuple *winning_bid;
    double price;
    LandManager *winner;

    new_bid = [new_sp parcelOffer];

    lp = [parcelsForSale removeFirst];
    lp_bids = [bids removeKey: lp];
    if(lp_bids == nil) {
      lp_bids = [List create: scratchZone];
    }
    [lp_bids addLast: [[[Tuple create: scratchZone]
			 setObj: new_sp]
			setDouble: new_bid]];

    [MiscFunc shuffleList: lp_bids];

    [MiscFunc mergeSort: lp_bids withSelector: M(getDouble)];

    if([FearlusArguments debugCoreMethods]) {
      id <Index> ix;
      Tuple *bid;

      printf("A: %s: List of bids for land parcel %u at (%d, %d):\n",
	     sel_get_name(_cmd), [lp getPIN], [lp getX], [lp getY]);

      for(ix = [lp_bids begin: scratchZone], bid = [ix next];
	  [ix getLoc] == Member;
	  bid = [ix next]) {
	id bidder = [bid getObj];

	if(bidder == new_sp) {
	  printf("\tNew land manager from subpopulation %s bid %g\n",
		 object_get_class_name(new_sp), [bid getDouble]);
	}
	else {
	  printf("\tLand manager %u bid %g\n",
		 [bidder getPIN], [bid getDouble]);
	}
      }
      [ix drop];
    }

    winning_bid = [lp_bids removeLast];
				// Assume ascending order
    if(![parameter vickrey] || [lp_bids getCount] == 0) {
				// First price sealed bid, or only one bid
				// (which has been removed from the list)
      price = [winning_bid getDouble];
    }
    else {			// Vickrey auction
      Tuple *runner_up = [lp_bids removeLast];

      price = [runner_up getDouble];

      [runner_up drop];
    }

    if([winning_bid getObj] == new_sp) {
      winner = [self createNewLandManager: new_sp initialWealth: price];

      if([FearlusArguments debugCoreMethods]) {
	printf("Winning bidder is new land manager %u from subpopulation %s",
	       [winner getPIN], object_get_class_name(new_sp));
      }
    }
    else {
      winner = [winning_bid getObj];

      if([FearlusArguments debugCoreMethods]) {
	printf("Winning bidder is land manager %u", [winner getPIN]);
      }
    }
    [winning_bid drop];

    if([FearlusArguments debugCoreMethods]) {
      printf(", price to pay is %g\n", price);
    }

    [winner addLandParcel: lp price: price];
    [lp setNextLandManagerTo: winner];

    [lp_bids deleteAll];
    [lp_bids drop];
  }
}

/*

chooseSupPopulation

Return a subpopulation chosen for a new land manager to belong to

*/

-(SubPopulation *)chooseSubPopulation {
  SubPopulation *sp = nil;
  double cumulativeProb;
  double choice;
  unsigned i;

  // Decide the sub-population of the land manager
  cumulativeProb = 0.0;
  choice = [uniformDblRand getDoubleWithMin: 0.0 withMax: 1.0];
  for(i = 0; i < [parameter nSubPopulations]; i++) {
    sp = [parameter subPopulation: i];
    cumulativeProb += [sp getProb];
    if(choice <= cumulativeProb) {
      break;
    }
  }
  if(choice > cumulativeProb) {
    fprintf(stderr, "%s -- Probabilities for sub-populations do not sum to 1.0"
	    " (%g). Aborting simulation.\n", sel_get_name(_cmd), 
	    cumulativeProb);
    abort();
  }
  // If we're debugging core methods, then print a message showing how the
  // choice was made.
  if([FearlusArguments debugCoreMethods]) {
    printf("A: %s: Sub-population of potential new land manager selected as "
	   "follows:\n\tSub-pop Probability (Cumulative)\n",
	   sel_get_name(_cmd));
    cumulativeProb = 0.0;
    for(i = 0; i < [parameter nSubPopulations]; i++) {
      cumulativeProb += [[parameter subPopulation: i] getProb];
      printf("\t%7u       %g (%g)", [[parameter subPopulation: i] getPIN],
	     [[parameter subPopulation: i] getProb], cumulativeProb);
      if([parameter subPopulation: i] == sp) {
	printf(" <-- choice = %g\n", choice);
      }
      else {
	printf("\n");
      }
    }
  }

  return sp;
}

/*

createNewLandManager

Create a new land manager, initialise it, and store it in the list of land
managers

*/

-(LandManager *)createNewLandManager {
  LandManager *lm;
  SubPopulation *sp;

  sp = [self chooseSubPopulation];
  lm = [self createNewLandManager: sp initialWealth: 0.0];

  return lm;
}

/*

createNewLandManager:initialWealth:

Create a new land manager with the specified initial wealth

*/

-(LandManager *)createNewLandManager: (SubPopulation *)sp
		       initialWealth: (double)money {
  LandManager *lm;

  lm = [[sp getLandManagerClass] create: [self getZone]];
  [lm setSubPopulation: sp];
  [sp addLandManager: lm];
  [lm setParameters: parameter];
  [lm initialiseWithEnvironment: environment
      landAllocator: self
      colour: [self getNextFreeColour]
      wealth: money];
				// Initialise the land manager
  [landManagers addLast: lm];
  barnCredit += money;		// wealth

  return(lm);
}

/*

getLandManagers

Return the list of land managers (who are currently in the game)

*/

-getLandManagers {
  return(landManagers);
}

/*

killLosers

Rather a brutal name for a method that moves land managers who have no land
parcels left onto the list of land managers who are no longer making decisions
about land parcels. Works by removing all land managers from the land manager
list one by one, and sorting them into two lists according to whether or not
they are staying in the game. Then copy the list of managers staying in the
game back into the land manager list.

*/

-(void)killLosers {
  id tempZone, lml;
  LandManager *lm;
  int nKilled;

  tempZone = [Zone create: scratchZone];
  lml = [List create: tempZone];
				// A list that will store land managers to
				// stay in the game
  // Loop through all the land managers sorting them into winners and losers
  nKilled = 0;
  while([landManagers getCount] > 0) {
    lm = [landManagers removeFirst];
    if([lm eligibleForRemovalFromGame]) {
      [deadLandManagers addLast: lm];
				// Remove the land manager from the game
      [justDeadLandManagers addLast: lm];
      // (^) remember the land managers who died this year so they can record
      //     their history. (Assume, therefore, that killLosers occurs before
      //     recordHistory in the schedule.)
      nKilled++;
      if([FearlusArguments debugAllocator]) {
	if(nKilled == 1) {
	  printf("A: Killing land manager(s): %u", [lm getPIN]);
	}
	else {
	  printf(", %u", [lm getPIN]);
	}
      }
      [lm kill];		// Tell the land manager they are out
      barnCredit -= [lm getBarn]; // should be wealth
      // Subtract the final wealth of the land manager from the barnCredit
      // -- effectively we are keeping track of the amount of wealth the land
      // allocator would have to pay to balance the books, as it were.
      [[lm getSubPopulation] removeLandManager: lm];
    }
    else {
      [lml addLast: lm];	// Keep this land manager
    }
  }
  if([FearlusArguments debugAllocator]) {
    if(nKilled == 0) {
      printf("A: No land managers to kill\n");
    }
    else {
      printf("\n");
    }
  }
  // Loop through the list of winners and add them back to the list of land
  // managers that are in the game.
  while([lml getCount] > 0) {
    lm = [lml removeFirst];
    [landManagers addLast: lm];
  }
  [lml drop];
  [tempZone drop];
}

/*

getNextFreeColour

For now, this method is a simple loop round the available colours from min to
max. In future, this should (a) ensure that sequential colours are as far apart
in colour space as possible (b) keep a record of dead land manager's colours, 
and assign them out.

*/

-(int)getNextFreeColour {
  if(++nextColour > maxColour) {
    nextColour = minColour;
  }
  return nextColour;
}

/*

writeHistory

Write the history of the land allocator and associated objects to a file

*/

-(void)writeHistory: (FILE *)fp {
  id iterator;
  LandManager *lm;

  fprintf(fp, "Type\tID\tSub-Pop ID\tYear\tParcels Gained\tAge\tBarn\t"
	  "Neighbourhood Weight\tNeighbour Information Loss\t"
	  "Habit Imitation Yield Threshold\tProbability of Imitative Strategy "
	  "Method\tInitial Strategy\tAbove Habit Threshold Strategy\t"
	  "Below Habit Threshold Imitative Strategy\t"
	  "Below Habit Threshold Non-Imitative Strategy%s", [FearlusArguments nl]);
  for(iterator = [landManagers begin: scratchZone], [iterator next];
      [iterator getLoc] == Member;
      [iterator next]) {
    lm = [iterator get];
    [lm printHistoryToFile: fp];
  }
  [iterator drop];
  /* Repeat for dead land managers */
  for(iterator = [deadLandManagers begin: scratchZone], [iterator next];
      [iterator getLoc] == Member;
      [iterator next]) {
    lm = [iterator get];
    [lm printHistoryToFile: fp];
    [deadDeadLandManagers addLast: lm];
  }
  [iterator drop];
  [deadLandManagers removeAll];
}

/*

recordHistory

Record the history (live land managers only!)

*/

-(void)recordHistory {
  [landManagers forEach: M(recordHistory)];
  [justDeadLandManagers forEach: M(recordHistory)];
  [justDeadLandManagers removeAll];
}

/*

getBarnCredit (should be wealth)

Return the barnCredit as a double (because I reckon the graphing tools probably
can't handle long longs). The barnCredit is the amount of wealth effectively
given to land managers through providing them with their initial land parcel
and paying off their debts when they die. It can be reduced if land managers
die in credit, which is possible for certain settings of the landParcelPrice.

*** The barnCredit is now a double anyway. The landParcelPrice and break even
threshold can both be non-integral (and still make sense). ***

*/

-(double)getBarnCredit {	// wealth
  return barnCredit;		// wealth
}

/*

getWealth

Return the sum of the wealths of the land managers minus the barnCredit, as a
double so that it can be graphed. This represents the amount of wealth created
during the simulation.

*/

-(double)getWealth {
  id lmi;
  double total;

  total = 0;
  for(lmi = [landManagers begin: scratchZone], [lmi next];
      [lmi getLoc] == Member;
      [lmi next]) {
    total += [[lmi get] getBarn]; // wealth
  }
  [lmi drop];
  total -= barnCredit;		// wealth
  return total;
}

/*

drop

Delete all items created by this object

*/

-(void)drop {
  [landManagers deleteAll];
  [deadLandManagers deleteAll];
  [landManagers drop];
  [deadLandManagers drop];
  [bids drop];
  [parcelsForSale drop];
  [super drop];
}

@end
