/*
    FEARLUS model0-6-8+ELMM0-2: NeighbouringOptimumStrategy.m
    Copyright (C) 1999-2002  Macaulay Institute

    This file is part of FEARLUS model0-6-8+ELMM0-2, an agent-based model of land use
    change.

    FEARLUS model0-6-8+ELMM0-2 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FEARLUS model0-6-8+ELMM0-2 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
/*

    FEARLUS model0-6-8+ELMM0-2 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FEARLUS model0-6-8+ELMM0-2 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
/*


Implementation for the NeighbouringOptimumStrategy object.

*/

#import "NeighbouringOptimumStrategy.h"
#import "SelectUseBucket.h"
#import "Environment.h"
#import "LandUse.h"
#import "LandParcel.h"
#import "LandManager.h"
#import "Parameter.h"
#import "FearlusArguments.h"
#import "BitString.h"

@implementation NeighbouringOptimumStrategy

/*

create:withLandManager:

Return a newly created RandomStrategy object, configured with the land manager
specified in the argument

*/

+(id <ImitativeStrategy, HistoricalStrategy>)create: aZone
	   withManager: (LandManager *)lmgr
	 andParameters: (Parameter *)p {
  NeighbouringOptimumStrategy *obj;

  obj = [super create: aZone];
  obj->lm = lmgr;
  obj->parameter = p;
  return obj;
}

/*

decideLandUseForParcel:

Return a land use for the parcel, chosen at random among those
appearing in the neighbourhood with equal maximum yield on the
decision parcel.

*/

-(LandUse *)decideLandUseForParcel: (LandParcel *)lp {
  SelectUseBucket *bucket;
  id parcelIndex, tempZone;
  LandParcel *myParcel, *neighbouringParcel;
  LandManager *neighbour;
  LandUse *lu;
  int i;
  double match;
  BitString *climate, *economy;

  climate = [[lp getEnvironment] getClimate];
  economy = [[lp getEnvironment] getEconomy];

  tempZone = [Zone create: scratchZone];
  bucket = [SelectUseBucket create: tempZone
			    withParameters: parameter
			    andLandUses: [[lp getEnvironment] getLandUses]];
  // Loop through land parcels owned by the land manager
  if([FearlusArguments debugCoreMethods]) {
    printf("S: %s: Using neighbouring optimum strategy of land manager"
	   " %u to determine land use for land parcel %u at (%d, %d)\n",
	   sel_get_name(_cmd), [lm getPIN], [lp getPIN], [lp getX], [lp getY]);
  }
  for(parcelIndex = [[lm getLandParcels] begin: tempZone], [parcelIndex next];
      [parcelIndex getLoc] == Member;
      [parcelIndex next]) {
    myParcel = [parcelIndex get];
    lu = [myParcel getLandUse];
    match = (double)[lu getMatchWithLandParcel: lp 
			climate: climate
			economy: economy];
    [bucket setScore: match inBucketForLandUse: lu];
    [myParcel incNImitations];
    if([FearlusArguments debugCoreMethods]) {
      printf("S: %s: Land parcel %u at (%d, %d) has land use %u. Score for "
	     "land use incremented by %g\n", sel_get_name(_cmd),
	     [myParcel getPIN], [myParcel getX], [myParcel getY],
	     [lu getPIN], 1.0);
    }
  }
  [parcelIndex drop];
  // Loop through the neighbours, and through their land parcels
  for(i = 0; (neighbour = [lm getNextNeighbour: i]); i |= 1) {
    if([FearlusArguments debugCoreMethods]) {
      printf("S: %s: Adding scores for land uses of land parcels belonging"
	     " to land manager %u\n", sel_get_name(_cmd), [neighbour getPIN]);
    }
    for(parcelIndex = [[neighbour getLandParcels] begin: tempZone],
	  [parcelIndex next];
	[parcelIndex getLoc] == Member;
	[parcelIndex next]) {
      neighbouringParcel = [parcelIndex get];
      lu = [neighbouringParcel getLandUse];
      match = (double)[lu getMatchWithLandParcel: lp 
			  climate: climate
			  economy: economy];
      [bucket setScore: [lm getNeighbourWeight] * match
	      inBucketForLandUse: lu];
      [neighbouringParcel incNImitations];
      if([FearlusArguments debugCoreMethods]) {
	printf("S: %s: Land parcel %u at (%d, %d) has land use %u. Score of "
	       "land use incremented by %g\n", sel_get_name(_cmd),
	       [neighbouringParcel getPIN], [neighbouringParcel getX],
	       [neighbouringParcel getY],
	       [lu getPIN],
	       [lm getNeighbourWeight]);
      }
    }
    [parcelIndex drop];
  }
  lu = [bucket getRandomOptimumChoice];
  [bucket drop];
  [tempZone drop];
  return lu;
}

@end
