/*
    FEARLUS model0-6-8+ELMM0-2: PositiveLandManager.m
    Copyright (C) 1999-2002  Macaulay Institute

    This file is part of FEARLUS model0-6-8+ELMM0-2, an agent-based model of land use
    change.

    FEARLUS model0-6-8+ELMM0-2 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FEARLUS model0-6-8+ELMM0-2 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
/*

    FEARLUS model0-6-8+ELMM0-2 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FEARLUS model0-6-8+ELMM0-2 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
/*


Implementation for the PositiveLandManager class.

*/

#import "PositiveLandManager.h"
#import "FearlusArguments.h"
#import "MiscFunc.h"
#import "LandParcel.h"
#import "LandAllocator.h"
#import "Parameter.h"

@implementation PositiveLandManager

/*

sellLandParcels

Add land parcels for transfer to the list. Remove these land parcels from the
list of self's land parcels. The land parcels to transfer are the N
lowest-yielding land parcels owned by the land manager such that:

barn + (N * [parameter landParcelPrice]) > 0 // wealth

If a land manager must sell all their land parcels to do this, then they are
retired.

Changes for ELMM 0-1:

All land parcels are put up for sale if the wealth is less than 0.

*/

-(void)sellLandParcels  {
  someLandParcelsLost = NO;
  allLandParcelsLost = NO;
  /* It is OK to set the numberOfLandParcelsGainedThisYear to zero when selling
     land parcels -- this will not wipe out any increments, because there can
     be no gaining of land parcels until it has been determined which are to
     be lost. */
  numberOfLandParcelsGainedThisYear = 0;
  // If the wealth is >= 0 we can leave this method without further ado
  if(barn > 0.0) {		// wealth
    if([FearlusArguments debugManagers]) {
      printf("M: Positive Land manager %u loses no land parcels this year "
	     "since barn %g > 0\n", pin, barn); // wealth
    }
    return;
  }

  // ELMM 0-1: All land parcels owned are put up for sale

  if([FearlusArguments debugManagers]) {
    printf("M: Land manager %u has negative barn %g, so puts all their parcels"
	   " up for sale\n", pin, barn); // wealth
  }

  while([landParcelsOwned getCount] > 0) {
    LandParcel *lp;

    lp = [landParcelsOwned removeFirst];
    [landAllocator addLandParcelForSale: lp];

    if([FearlusArguments debugManagers]) {
      printf("M: Land manager %u puts parcel %u at (%d, %d) up for sale\n",
	     pin, [lp getPIN], [lp getX], [lp getY]);
    }

    numberOfLandParcelsGainedThisYear--;
    someLandParcelsLost = YES;
  }
  allLandParcelsLost = YES;
}

/*

eligibleForNewLandParcels

A land manager is eligible for new land parcels if their wealth is greater than
or equal to the landParcelPrice.

*/

-(BOOL)eligibleForNewLandParcels {
  if(barn >= landOfferThreshold) { // wealth
    return(YES);
  }
  else {
    return(NO);
  }
}


@end
