/********************************************************************************/
/* Name : myAzmContinuousCorelogram                                            */
/* Goal : Calcul the Correlogram of input continuous Data for a specific azimuth */
/*        and save it in a textfile                                             */
/* in   :       inVal : Matrice with the binaries values       Matrix converted to vector by putting one row after another                       */
/*              inX: Matrix containing the x coordinates for each scale        */
/*              inY: Matrix containing the y coordinates for each scale         */
/*               hlag  : lenght of thhe unit lag               1                */
/*              Nblag : number of lags                         width/2                 */
/*              tol  : Distance Tolerence                      0.5                 */
/*              ind  : beta index used                         not used                 */
/*              azimuth : azimuth angle of direction           0                  */
/*              atol  : azimuth tolerence                      180                 */
/*              hlag  : lenght of thhe unit lag                1                 */
/*              bandwith  : Maximum  bandwith                  set to a large number                 */
/*              FileName  : Name for the ouput file                             */
/********************************************************************************/
int TForm1::myAzmContinusCorrelogram(const vector<float> & inVal,const vector<float> & inX,const vector<float> & inY,vector<float> &outMatrix,float hlag,int NbLag,float tol,float azimuth, float atol,float bandwith,AnsiString FileName)
{
   float Max;
   float dist,Ro;
   int LagNumber;
   int dx,dy;
   float xzm,yzm,cstol,azm,dcazm,band;
   double tempres,rox,roy;
   double median;
   int res=0;

   //if enough place reinit all the matrix mecessary
   if(NbLag+1<MAXSIMU*MAXNBLAG)
   {
      outMatrix.clear();
      outMatrix.resize(NbLag+1);
      d.clear();
      d.resize(NbLag+1);
      nbvar.clear();
      nbvar.resize(NbLag+1);

      //Matrix  to save the alle the value for each Lag
      double **ValuesMatrixHead=new double *[NbLag+1];
      double **ValuesMatrixTail=new double *[NbLag+1];

      for (int i=0;i<NbLag+1;i++)
      {
         ValuesMatrixHead[i]=new double[10*MAXVALUES];
         ValuesMatrixTail[i]=new double[10*MAXVALUES];
         for (int j=0;j<10*MAXVALUES;j++)
         {
             ValuesMatrixHead[i][j]=0;
             ValuesMatrixTail[i][j]=0 ;
         }
      }


      ofstream f;
      f.open(FileName.c_str()) ;
      //Init Matrix
      for (int i=0;i<NbLag+1;i++)
      {
        outMatrix[i]=0;
        nbvar[i]=-1;
        d[i]=0;
      }
      int size=inX.size();
      if((int)inY.size()==size && (int)inVal.size()==size)
      {
      //the maximum distance defined by the user
         Max=hlag*((float)NbLag+tol)*hlag*((float)NbLag+tol);
         for (int ih=0;ih<size;ih++)     // x coordinate of the head
         {
            for (int it=0;it<size;it++)  //x coordinate of the tail
            {
   //calculate the distance between 2 pairs
               dx=inX[it]-inX[ih];
               dy= inY[it]-inY[ih];
               dist=dx*dx+dy*dy;
               if(dist>0 && dist<Max) //if we are not below the highest lag
               {
                  dist=sqrt(dist);
                  LagNumber=0;
   //Determine wich lag this is
                  for (int k=1;k<NbLag+1;k++)
                  {
                     if(dist>=(float)k*hlag-tol && dist<=(float)k*hlag+tol)
                     {
                        LagNumber=k;
                        k=NbLag+1;
                     }
                  }
                  if (LagNumber)
                  {
   //check for an acceptable azimuth angle
                     azm=(M_PI*(90-(azimuth)))/180.0;
                     xzm=cos(azm);
                     yzm=sin(azm);
                     cstol=cos(atol*M_PI/180.0);
                     dcazm=(xzm*dx+yzm*dy)/dist;
                     if(fabs(dcazm)>=cstol)
                     {
   //Check the horizontal bandwitch criteria
  //(maximum deviation perpendicular to the specified direction azimuth
                        band=dx*yzm-dy*xzm;
                        if (abs(band)<=bandwith )
                        {
  //Compute the appropriate similarity measure
                           nbvar[(LagNumber-1)]++;
                           d[(LagNumber-1)]+=dist;
                           ValuesMatrixHead[(LagNumber-1)][nbvar[(LagNumber-1)]]=(double)inVal[ih];;
                           ValuesMatrixTail[(LagNumber-1)][nbvar[(LagNumber-1)]]=(double)inVal[it];
                        }
                     }
                  }
               }
            }
         }
     }
  // for each Lag calculta the correlation coefficient
     for(int k=0;k<NbLag+1 ;k++)
     {
        tempres=gsl_stats_covariance (ValuesMatrixHead[k], 1, ValuesMatrixTail[k],1,nbvar[k]+1);
        rox=gsl_stats_sd (ValuesMatrixHead[k],1,nbvar[k]+1);
        roy=gsl_stats_sd (ValuesMatrixTail[k],1,nbvar[k]+1);
        if(rox && roy)
        {
           tempres/=(rox*roy);
        }
        else tempres=0;
        outMatrix[k]=(float)tempres;
     }

   //Save the result in a text file
      f<<"       Correlogram from continuous values"<<endl;
      f<<"colums are: distance (h),gamma, number of pairs"<<endl<<endl;
      f<<"lag distance is "<<hlag<<" and there are "<<NbLag<< " lags"<< " lags  azimuth="<<azimuth<<" azimuth tolerence="<<atol<<endl<<endl  ;
      for (int k=0;k<NbLag;k++)
      {
        if(nbvar[k])
        {
           d[k]/=(float)nbvar[k];
        }
        f<<d[k]<<" , "<<outMatrix[k]<<" , "<<nbvar[k]/2<<endl;
      }

      f.close();
      kind=5;
      res=1;
      mydelete(&ValuesMatrixHead,NbLag+1);
      mydelete(&ValuesMatrixTail,NbLag+1);

   }
   return res;
}
